// $Id: textmarker_marker.js,v 1.1.2.3 2010/08/12 09:51:26 kartelusm Exp $

//This library displays a text marker.
 
function TextMarker(coordinates, options) {
  this.point = coordinates;
  options = options || {};
  this.tag = options.text || "";
  if (options.text) {
	  options.text = undefined;
  }
  
  this.infoText = options.infoText;
  if (options.infoText) {
	  options.infoText = undefined;
  }
  
  this.options = options;
  
  this.minFontSize = options.minFontSize || "12";
  this.fontSizeRatio = options.fontSizeRatio || 0.25;
  
  this.wrapperID = options.id || "textmarker";
  this.weight = options.weight || 1;
  this.fontStyle = options.fontStyle;
  this.markerColor = options.markerColor;
  
  this.icon = options.icon;
  this.icon.shadow = null;
  this.icon.iconSize  = new GSize(0,0);
  this.icon.shadowSize  = new GSize(0,0);
  this.icon.image = "";
  this.icon.iconAnchor = new GPoint(0,0);

  GMarker.apply(this, arguments);
}

// Create GMarker

TextMarker.prototype = new GMarker(new GLatLng(0, 0));

// Initialize the marker
TextMarker.prototype.initialize = function (m) {
  GMarker.prototype.initialize.apply(this, arguments);
  this.map = m;
  //  Make the wrapper 
  this.wrapper = document.createElement("div");
  // IE<8 exception
  if (this.isIE() === true && this.getInternetExplorerVersion() < 8) {
    this.wrapper.setAttribute('className', this.wrapperID + ' level' + this.weight); // ADD CLASSNAME BASED ON WEIGHT 
  }
  else {
    this.wrapper.setAttribute('class', this.wrapperID + ' level' + this.weight); // ADD CLASSNAME BASED ON WEIGHT 
  }
  m.getPane(G_MAP_MARKER_PANE).appendChild(this.wrapper);
  this.wrapper.style.zIndex = GOverlay.getZIndex(this.point.lat());
  this.wrapper.style.position = "absolute";
  this.wrapper.style.visibility = "hidden";
  
    this.displayTxt();
    
  // Events 
  GEvent.bindDom(this.wrapper, "click", this, function () {    
	this.openInfoWindowHtml(this.infoText, { maxWidth: this.map.getSize().width/2 });
  });
};

//ignore
TextMarker.prototype.copy = function () {
  this.options.text = this.tag;
  return new TextMarker(this.point, this.options);
};

TextMarker.prototype.redraw = function(f) {
	GMarker.prototype.redraw.apply(this, arguments);
	if (f) {
		this.showText();
		this.point = this.getLatLng();
		this.wrapper.style.zIndex = GOverlay.getZIndex(this.point.lat());
	}
};

//return size of html elements

TextMarker.prototype.buildHtml = function (el) {
  var mapWrapper = this.map.getContainer();
  var placeholder = document.createElement("div");  
  placeholder.style.display = "inline";
  placeholder.style.position = "absolute";
  placeholder.style.fontSize = el.style.fontSize;
  placeholder.innerHTML = el.innerHTML;  
  mapWrapper.appendChild(placeholder);
  
  var fullSize = new GSize(1, 1); 
  fullSize.width = placeholder.offsetWidth;
  fullSize.height = placeholder.offsetHeight;
  mapWrapper.removeChild(placeholder);
  
  return fullSize;
};

//Create wrapper with text 

TextMarker.prototype.displayTxt = function () {

//Make text wrapper
this.bodywrapper = document.createElement("div");
// IE<8 exception
if (this.isIE() === true && this.getInternetExplorerVersion() < 8) {
  this.bodywrapper.setAttribute('className', this.wrapperID + '_body');
}
else {
  this.bodywrapper.setAttribute('class', this.wrapperID + '_body');
}
this.bodywrapper.style.position = "absolute";
this.bodywrapper.style.backgroundColor = "transparent";
this.bodywrapper.style.overflow = "hidden";
this.wrapper.appendChild(this.bodywrapper);
};

//Redraws the text.

TextMarker.prototype.redrawTxt = function (text) {  
  this.bodywrapper.style.fontSize = (((this.weight-1) * this.fontSizeRatio + 1) * this.minFontSize) + "px";
  this.bodywrapper.style.fontStyle = this.fontStyle;
  if (this.markerColor != 'css')
	this.bodywrapper.style.color = this.markerColor;
  while (this.bodywrapper.firstChild) {
    this.bodywrapper.removeChild(this.bodywrapper.firstChild);
  }
  this.bodywrapper.innerHTML = text;
  if (this.isIE() === false && this.bodywrapper.hasChildNodes) {
    if (this.bodywrapper.firstChild.nodeType === 1) {
      this.bodywrapper.firstChild.style.margin = 0;
    }
  }	
  this.size = this.buildHtml(this.bodywrapper);
  this.wrapper.style.width = this.size.width + "px";
  this.wrapper.style.height = this.size.height + "px";
  var pxPos = this.map.fromLatLngToDivPixel(this.point);
   
  this.wrapper.style.left = (pxPos.x - (this.size.width/2)) + "px";
  this.wrapper.style.top = (pxPos.y - this.size.height/2) + "px";
};

//Shows the marker.

TextMarker.prototype.show = function () {
GMarker.prototype.show.apply(this, arguments);
};

//Hides the marker.

TextMarker.prototype.hide = function () {
GMarker.prototype.hide.apply(this, arguments);
this.wrapper.style.visibility = "hidden";
};

TextMarker.prototype.remove = function () {
	  GEvent.clearInstanceListeners(this.wrapper);
	  while (this.wrapper.firstChild) {
	    this.wrapper.removeChild(this.wrapper.firstChild);
	  }
	  this.wrapper.parentNode.removeChild(this.wrapper);
	  GMarker.prototype.remove.apply(this, arguments);
	  delete arguments.callee;
};

//Shows the marker and the text.

TextMarker.prototype.showText = function () {
  this.show();
  
  this.redrawTxt(this.tag);
    
  var iw = this.map.getInfoWindow();
  if (!iw.isHidden() || this.isNull(this.tag)) {
    return;
  }
  this.wrapper.style.visibility = "visible";
};

// detects null,null string and undefined

TextMarker.prototype.isNull = function (s) {
  if (!s && s !== 0 || s === null || s === "" || s === undefined || typeof s === "undefined") {
    return true;
  }
  return false;
};

// detects IE

TextMarker.prototype.isIE = function () {
  return (navigator.userAgent.toLowerCase().indexOf('msie') !== -1) ? true : false;
};

// Returns the version of Internet Explorer or a -1

TextMarker.prototype.getInternetExplorerVersion = function() {
  var rv = -1; // Return value assumes failure.
  if (navigator.appName == 'Microsoft Internet Explorer') {
    var ua = navigator.userAgent;
    var re  = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
    if (re.exec(ua) != null)
      rv = parseFloat( RegExp.$1 );
  }
  return rv;
}